#include <QtCore>
#include <QtGui>

#include "psiplugin.h"
#include "eventfilter.h"
#include "optionaccessor.h"
#include "optionaccessinghost.h"

#define constUcolor "usercolor"
#define constTcolor "tagcolor"
#define constMcolor "idcolor"
#define constUbold "userbold"
#define constTbold "tagbold"
#define constMbold "idbold"
#define constUitalic "useritalic"
#define constTitalic "tagitalic"
#define constMitalic "iditalic"
#define constUunderline "userunderline"
#define constTunderline "tagunderline"
#define constMunderline "idunderline"

class JuickPlugin : public QObject, public PsiPlugin, public EventFilter, public OptionAccessor
{
	Q_OBJECT
        Q_INTERFACES(PsiPlugin EventFilter OptionAccessor)

public:
        JuickPlugin();
	virtual QString name() const;
	virtual QString shortName() const;
	virtual QString version() const;
	virtual QWidget* options();
	virtual bool enable();
	virtual bool disable();
        virtual void applyOptions();
        virtual void restoreOptions();
        virtual bool processEvent(int account, QDomElement& e);
        virtual bool processMessage(int account, const QString& fromJid, const QString& body, const QString& subject);
        // OptionAccessor
        virtual void setOptionAccessingHost(OptionAccessingHost* host);
        virtual void optionChanged(const QString& option);
private:
        bool enabled;
        QCheckBox *ub_button;
        QCheckBox *ui_button;
        QCheckBox *uu_button;
        QCheckBox *tb_button;
        QCheckBox *ti_button;
        QCheckBox *tu_button;
        QCheckBox *mb_button;
        QCheckBox *mi_button;
        QCheckBox *mu_button;
        QToolButton *uc_button;
        QToolButton *tc_button;
        QToolButton *mc_button;
        QWidget *optionsWid;
        QColor ucolor;
        QColor tcolor;
        QColor mcolor;
        OptionAccessingHost* psiOptions;
        bool ubold;
        bool tbold;
        bool mbold;
        bool uitalic;
        bool titalic;
        bool mitalic;
        bool uunderline;
        bool tunderline;
        bool munderline;
private slots:
        void chooseColor(QAbstractButton*);
};

Q_EXPORT_PLUGIN(JuickPlugin);

JuickPlugin::JuickPlugin():enabled(false){
    ucolor = QColor(0, 85, 255);
    tcolor = QColor(131, 145, 145);
    mcolor = QColor(87, 165, 87);
    ubold = true;
    tbold = false;
    mbold = false;
    uitalic = false;
    titalic = true;
    mitalic = false;
    uunderline = false;
    tunderline = false;
    munderline = true;
}
QString JuickPlugin::name() const
{
        return "Juick Plugin";
}

QString JuickPlugin::shortName() const
{
        return "juick";
}

QString JuickPlugin::version() const
{
        return "0.1.1";
}

QWidget* JuickPlugin::options()
{
        if (!enabled) {
                return 0;
        }
        QWidget *optionsWid = new QWidget();
        QGridLayout *layout= new QGridLayout(optionsWid);
        layout->addWidget(new QLabel(tr("@username"),optionsWid),1,0);
        layout->addWidget(new QLabel(tr("*tag"),optionsWid),2,0);
        layout->addWidget(new QLabel(tr("#message id"),optionsWid),3,0);
        layout->addWidget(new QLabel(tr("bold"),optionsWid),0,1);
        layout->addWidget(new QLabel(tr("italic"),optionsWid),0,2);
        layout->addWidget(new QLabel(tr("underline"),optionsWid),0,3);
        layout->addWidget(new QLabel(tr("color"),optionsWid),0,4);
        ub_button = new QCheckBox(optionsWid);
        ui_button = new QCheckBox(optionsWid);
        uu_button = new QCheckBox(optionsWid);
        tb_button = new QCheckBox(optionsWid);
        ti_button = new QCheckBox(optionsWid);
        tu_button = new QCheckBox(optionsWid);
        mb_button = new QCheckBox(optionsWid);
        mi_button = new QCheckBox(optionsWid);
        mu_button = new QCheckBox(optionsWid);
        uc_button = new QToolButton(optionsWid);
        tc_button = new QToolButton(optionsWid);
        mc_button = new QToolButton(optionsWid);
        layout->addWidget(ub_button,1,1);
        layout->addWidget(ui_button,1,2);
        layout->addWidget(uu_button,1,3);
        layout->addWidget(uc_button,1,4);
        layout->addWidget(tb_button,2,1);
        layout->addWidget(ti_button,2,2);
        layout->addWidget(tu_button,2,3);
        layout->addWidget(tc_button,2,4);
        layout->addWidget(mb_button,3,1);
        layout->addWidget(mi_button,3,2);
        layout->addWidget(mu_button,3,3);
        layout->addWidget(mc_button,3,4);
        QButtonGroup *b_color = new QButtonGroup;
        b_color->addButton(uc_button);
        b_color->addButton(tc_button);
        b_color->addButton(mc_button);
        uc_button->setStyleSheet(QString("background-color: %1;").arg(ucolor.name()));
        tc_button->setStyleSheet(QString("background-color: %1;").arg(tcolor.name()));
        mc_button->setStyleSheet(QString("background-color: %1;").arg(mcolor.name()));
        uc_button->setProperty("psi_color",ucolor);
        tc_button->setProperty("psi_color",tcolor);
        mc_button->setProperty("psi_color",mcolor);
        ub_button->setChecked(ubold);
        tb_button->setChecked(tbold);
        mb_button->setChecked(mbold);
        ui_button->setChecked(uitalic);
        ti_button->setChecked(titalic);
        mi_button->setChecked(mitalic);
        uu_button->setChecked(uunderline);
        tu_button->setChecked(tunderline);
        mu_button->setChecked(munderline);
        connect(b_color, SIGNAL(buttonClicked(QAbstractButton*)), SLOT(chooseColor(QAbstractButton*)));
        return optionsWid;
}

bool JuickPlugin::enable()
{
        enabled = true;
        QVariant vUcolor(ucolor);
        vUcolor = psiOptions->getPluginOption(constUcolor);
        if (!vUcolor.isNull()) {
                ucolor = vUcolor.toString();

        }
        QVariant vTcolor(tcolor);
        vTcolor = psiOptions->getPluginOption(constTcolor);
        if (!vTcolor.isNull()) {
                tcolor = vTcolor.toString();
        }
        QVariant vMcolor(mcolor);
        vMcolor = psiOptions->getPluginOption(constMcolor);
        if (!vMcolor.isNull()) {
                mcolor = vMcolor.toString();
        }
        //bold
        QVariant vUbold(ubold);
        vUbold = psiOptions->getPluginOption(constUbold);
        if (!vUbold.isNull()) {
                ubold = vUbold.toBool();

        }
        QVariant vTbold(tbold);
        vTbold = psiOptions->getPluginOption(constTbold);
        if (!vTbold.isNull()) {
                tbold = vTbold.toBool();
        }
        QVariant vMbold(mbold);
        vMbold = psiOptions->getPluginOption(constMbold);
        if (!vMbold.isNull()) {
                mbold = vMbold.toBool();
        }
        //italic
        QVariant vUitalic(uitalic);
        vUitalic = psiOptions->getPluginOption(constUitalic);
        if (!vUitalic.isNull()) {
                uitalic = vUitalic.toBool();

        }
        QVariant vTitalic(titalic);
        vTitalic = psiOptions->getPluginOption(constTitalic);
        if (!vTitalic.isNull()) {
                titalic = vTitalic.toBool();
        }
        QVariant vMitalic(mitalic);
        vMitalic = psiOptions->getPluginOption(constMitalic);
        if (!vMitalic.isNull()) {
                mitalic = vMitalic.toBool();
        }
        //underline
        QVariant vUunderline(uunderline);
        vUunderline = psiOptions->getPluginOption(constUunderline);
        if (!vUunderline.isNull()) {
                uunderline = vUunderline.toBool();

        }
        QVariant vTunderline(tunderline);
        vTunderline = psiOptions->getPluginOption(constTunderline);
        if (!vTunderline.isNull()) {
                tunderline = vTunderline.toBool();
        }
        QVariant vMunderline(munderline);
        vMunderline = psiOptions->getPluginOption(constMunderline);
        if (!vMunderline.isNull()) {
                munderline = vMunderline.toBool();
        }
	return true;
}

bool JuickPlugin::disable()
{
        enabled = false;
	return true;
}

void JuickPlugin::applyOptions(){
        if (!enabled || uc_button == 0 || tc_button == 0 || mc_button == 0){
            return;
        }
        ucolor = uc_button->property("psi_color").value<QColor>();
        tcolor = tc_button->property("psi_color").value<QColor>();
        mcolor = mc_button->property("psi_color").value<QColor>();
        QVariant vUcolor(ucolor);
        psiOptions->setPluginOption(constUcolor, vUcolor);
        QVariant vTcolor(tcolor);
        psiOptions->setPluginOption(constTcolor, vTcolor);
        QVariant vMcolor(mcolor);
        psiOptions->setPluginOption(constMcolor, vMcolor);
        //bold
        ubold = ub_button->isChecked();
        tbold = tb_button->isChecked();
        mbold = mb_button->isChecked();
        QVariant vUbold(ubold);
        psiOptions->setPluginOption(constUbold, vUbold);
        QVariant vTbold(tbold);
        psiOptions->setPluginOption(constTbold, vTbold);
        QVariant vMbold(mbold);
        psiOptions->setPluginOption(constMbold, vMbold);
        //italic
        uitalic = ui_button->isChecked();
        titalic = ti_button->isChecked();
        mitalic = mi_button->isChecked();
        QVariant vUitalic(uitalic);
        psiOptions->setPluginOption(constUitalic, vUitalic);
        QVariant vTitalic(titalic);
        psiOptions->setPluginOption(constTitalic, vTitalic);
        QVariant vMitalic(mitalic);
        psiOptions->setPluginOption(constMitalic, vMitalic);
        //underline
        uunderline = uu_button->isChecked();
        tunderline = tu_button->isChecked();
        munderline = mu_button->isChecked();
        QVariant vUunderline(uunderline);
        psiOptions->setPluginOption(constUunderline, vUunderline);
        QVariant vTunderline(tunderline);
        psiOptions->setPluginOption(constTunderline, vTunderline);
        QVariant vMunderline(munderline);
        psiOptions->setPluginOption(constMunderline, vMunderline);
}

void JuickPlugin::restoreOptions(){
        if (!enabled || uc_button == 0 || tc_button == 0 || mc_button == 0){
            return;
        }
        QVariant vUcolor(ucolor);
        vUcolor = psiOptions->getPluginOption(constUcolor);
        if (!vUcolor.isNull()) {
                uc_button->setStyleSheet(QString("background-color: %1;").arg(vUcolor.toString()));
        }
        QVariant vTcolor(tcolor);
        vTcolor = psiOptions->getPluginOption(constTcolor);
        if (!vTcolor.isNull()) {
                tc_button->setStyleSheet(QString("background-color: %1;").arg(vTcolor.toString()));
        }
        QVariant vMcolor(ucolor);
        vMcolor = psiOptions->getPluginOption(constMcolor);
        if (!vMcolor.isNull()) {
                mc_button->setStyleSheet(QString("background-color: %1;").arg(vMcolor.toString()));
        }
        //bold
        QVariant vUbold(ubold);
        vUbold = psiOptions->getPluginOption(constUbold);
        if (!vUbold.isNull()) {
                ub_button->setChecked(vUbold.toBool());
        }
        QVariant vTbold(tbold);
        vTbold = psiOptions->getPluginOption(constTbold);
        if (!vTbold.isNull()) {
                tb_button->setChecked(vTbold.toBool());
        }
        QVariant vMbold(mbold);
        vMbold = psiOptions->getPluginOption(constMbold);
        if (!vMbold.isNull()) {
                mb_button->setChecked(vMbold.toBool());
        }
        //italic
        QVariant vUitalic(uitalic);
        vUitalic = psiOptions->getPluginOption(constUitalic);
        if (!vUitalic.isNull()) {
                ui_button->setChecked(vUitalic.toBool());
        }
        QVariant vTitalic(titalic);
        vTitalic = psiOptions->getPluginOption(constTitalic);
        if (!vTitalic.isNull()) {
                ti_button->setChecked(vTitalic.toBool());
        }
        QVariant vMitalic(mitalic);
        vMitalic = psiOptions->getPluginOption(constMitalic);
        if (!vMitalic.isNull()) {
                mi_button->setChecked(vMitalic.toBool());
        }
        //underline
        QVariant vUunderline(uunderline);
        vUunderline = psiOptions->getPluginOption(constUunderline);
        if (!vUunderline.isNull()) {
                uu_button->setChecked(vUunderline.toBool());
        }
        QVariant vTunderline(tunderline);
        vTunderline = psiOptions->getPluginOption(constTunderline);
        if (!vTunderline.isNull()) {
                tu_button->setChecked(vTunderline.toBool());
        }
        QVariant vMunderline(munderline);
        vMunderline = psiOptions->getPluginOption(constMunderline);
        if (!vMunderline.isNull()) {
                mu_button->setChecked(vMunderline.toBool());
        }
}

bool JuickPlugin::processMessage(int account, const QString& fromJid, const QString& body, const QString& subject){
    Q_UNUSED(account);
    Q_UNUSED(fromJid);
    Q_UNUSED(body);
    Q_UNUSED(subject);
    return false;
}
bool JuickPlugin::processEvent(int account, QDomElement& e){
    Q_UNUSED(account);
    if (!enabled){
        return false;
    }
    //juick
    QString juick("juick@juick.com");
   if (e.childNodes().at(3).firstChild().nodeValue() == juick){
       //добавляем переводстроки для обработки ссылок и номеров сообщений в конце сообщения
        QString msg = e.lastChild().firstChild().firstChild().nodeValue()+"\n";
        QDomElement element =  e.ownerDocument().createElement("html");
        element.setAttribute("xmlns","http://jabber.org/protocol/xhtml-im");
        QDomElement body = e.ownerDocument().createElement("body");
        body.setAttribute("xmlns","http://www.w3.org/1999/xhtml");
        //int pos = msg.indexOf(":");
        int pos = 0;
        //сначала должен идти ник
        QRegExp start("^(@\\S*|Reply by @\\S*){1}:");
        //if (msg.at(0)=='@' && pos != -1){
        if (start.indexIn(msg, 0) != -1){
            //получаем юзернэйм
            //QString username = msg.left(pos);
            QString username = start.cap(1);
            if (username.at(0) == 'R'){
                pos = username.indexOf("@");
                body.appendChild(e.ownerDocument().createTextNode(username.left(pos)));
                username = username.mid(pos,username.size());

            }
            pos = start.matchedLength()-1;
            QString userStyle("");
            if (ubold) {
                userStyle += "font-weight: bold;";
            }
            if (uitalic) {
                userStyle += "font-style: italic;";
            }
            if (!uunderline) {
                userStyle += "text-decoration: none;";
            }
            userStyle += "color: " + ucolor.name();
            QDomElement user = e.ownerDocument().createElement("a");
            user.setAttribute("style",userStyle);
            user.setAttribute("href",QString("xmpp:%1?message;type=chat;body=%2+").arg(juick).arg(username));
            user.appendChild(e.ownerDocument().createTextNode(username));
            body.appendChild(user);
            body.appendChild(e.ownerDocument().createTextNode(":"));
            msg = msg.right(msg.size()-1-pos);
            QRegExp rx("^\\s(?!\\*\\S+\\*)(\\*\\S+)");
            //получаем список тегов
            QString tagStyle("");
            if (tbold) {
                tagStyle += "font-weight: bold;";
            }
            if (titalic) {
                tagStyle += "font-style: italic;";
            }
            if (!tunderline) {
                tagStyle += "text-decoration: none;";
            }
            tagStyle += "color: " + tcolor.name();
            for (int i=0; i < 5; ++i){
                if (rx.indexIn(msg, 0) != -1){
                    QDomElement ahref = e.ownerDocument().createElement("a");
                    ahref.setAttribute("style",tagStyle);
                    ahref.setAttribute("href",QString("xmpp:%1?message;type=chat;body=%2").arg(juick).arg(rx.cap(1)));
                    ahref.appendChild(e.ownerDocument().createTextNode(rx.cap(1)));
                    body.appendChild(ahref);
                    msg = msg.right(msg.size() - rx.matchedLength());
                } else {
                    break;
                }
            }
            //разбиваем на элементы освшееся сообщения и выделяем их
            int new_pos = 0;
            pos = 0;
            QRegExp regx("(\\s+)(#\\d*|#\\d*/\\d*|@\\S*|_\\S*_|\\*\\S*\\*|/\\S*/|http://\\S*|ftp://\\S*){1}(\\s+)");
            QString idStyle("");
            if (mbold) {
                idStyle += "font-weight: bold;";
            }
            if (mitalic) {
                idStyle += "font-style: italic;";
            }
            if (!munderline) {
                idStyle += "text-decoration: none;";
            }
            idStyle += "color: " + mcolor.name();
            while ((new_pos = regx.indexIn(msg, pos)) != -1){
                foreach(QChar space,msg.mid(pos,new_pos-pos+1)){
                    if (space == '\n'){
                        body.appendChild(e.ownerDocument().createElement("br"));
                    } else {
                        body.appendChild(e.ownerDocument().createTextNode(space));
                    }
                }
                QString seg = regx.cap(2);
                switch (seg.at(0).toAscii()){
                    case '#':{
                            QDomElement ahref = e.ownerDocument().createElement("a");
                            ahref.setAttribute("style",idStyle);
                            ahref.setAttribute("href",QString("xmpp:%1?message;type=chat;body=%2").arg(juick).arg(seg.replace("#","%23")));
                            ahref.appendChild(e.ownerDocument().createTextNode(seg.replace("%23","#")));
                            body.appendChild(ahref);
                            break;}
                    case '@':{
                            QString lastSymbol("");
                            if ((lastSymbol = seg.right(1)) == ":"){
                                seg = seg.left(seg.size() - 1);
                            }
                            QDomElement ahref = e.ownerDocument().createElement("a");
                            ahref.setAttribute("style",userStyle);
                            ahref.setAttribute("href",QString("xmpp:%1?message;type=chat;body=%2+").arg(juick).arg(seg));
                            ahref.appendChild(e.ownerDocument().createTextNode(seg));
                            body.appendChild(ahref);
                            if (lastSymbol == ":"){
                                body.appendChild(e.ownerDocument().createTextNode(":"));
                            }
                            break;}
                    case '*':{
                            QDomElement bold = e.ownerDocument().createElement("b");
                            bold.appendChild(e.ownerDocument().createTextNode(seg.mid(1,seg.size()-2)));
                            body.appendChild(bold);
                            break;}
                    case '_':{
                            QDomElement under = e.ownerDocument().createElement("u");
                            under.appendChild(e.ownerDocument().createTextNode(seg.mid(1,seg.size()-2)));
                            body.appendChild(under);
                            break;}
                    case '/':{
                            QDomElement italic = e.ownerDocument().createElement("i");
                            italic.appendChild(e.ownerDocument().createTextNode(seg.mid(1,seg.size()-2)));
                            body.appendChild(italic);
                            break;}
                    case 'h':
                    case 'f':{
                            QDomElement ahref = e.ownerDocument().createElement("a");
                            ahref.setAttribute("href",seg);
                            ahref.appendChild(e.ownerDocument().createTextNode(seg));
                            body.appendChild(ahref);
                            break;}
                    default:{}
                }
                pos = new_pos+regx.matchedLength()-regx.cap(3).size();
                new_pos = pos;
            }
            body.appendChild(e.ownerDocument().createTextNode( msg.right(msg.size()-pos)));
            body.appendChild(e.ownerDocument().createElement("br"));
            element.appendChild(body);
            e.lastChild().appendChild(element);

        }
    }
    return false;
}
void JuickPlugin::chooseColor(QAbstractButton* button)
{
        QColor c;
        c = button->property("psi_color").value<QColor>();
        c = QColorDialog::getColor(c, new QWidget());
        if(c.isValid()) {
                button->setProperty("psi_color", c);
                button->setStyleSheet(QString("background-color: %1").arg(c.name()));
                //HACK
                ub_button->toggle();
                ub_button->toggle();

        }
}
void JuickPlugin::setOptionAccessingHost(OptionAccessingHost* host)
{
        psiOptions = host;
}

void JuickPlugin::optionChanged(const QString& option)
{
        Q_UNUSED(option);
}
#include "juickplugin.moc"
